local _G = _G
local pairs = pairs
local ipairs = ipairs
local remove = table.remove
local insert = table.insert
local max = math.max
local min = math.min
local abs = math.abs
local atan2 = math.atan2
local sin = math.sin
local cos = math.cos
local type = type
local select = select
local next = next
local GetTime = GetTime
local deg2rad = math.pi/180.0
local pi = math.pi

local Core=AVR

AVR3D = {Embed = Core.Embed}
local T=AVR3D

local function setCoords(t, A, B, C, D, E, F)
	local det = A*E - B*D;
	local ULx, ULy, LLx, LLy, URx, URy, LRx, LRy;
	
	ULx, ULy = ( B*F - C*E ) / det, ( -(A*F) + C*D ) / det;
	LLx, LLy = ( -B + B*F - C*E ) / det, ( A - A*F + C*D ) / det;
	URx, URy = ( E + B*F - C*E ) / det, ( -D - A*F + C*D ) / det;
	LRx, LRy = ( E - B + B*F - C*E ) / det, ( -D + A -(A*F) + C*D ) / det;
	
	local mi=min(ULx,ULy,LLx,LLy,URx,URy,LRx,LRy)
	local ma=max(ULx,ULy,LLx,LLy,URx,URy,LRx,LRy)
--	if mi<-1000 then print("min="..mi) end
--	if ma>1000 then print("max="..ma) end
	
	t:SetTexCoord(ULx, ULy, LLx, LLy, URx, URy, LRx, LRy);
end

local function matrixMultiply(a,b)
	return {a[1]*b[1]+a[2]*b[4]+a[3]*b[7],  a[1]*b[2]+a[2]*b[5]+a[3]*b[8], a[1]*b[3]+a[2]*b[6]+a[3]*b[9],
	        a[4]*b[1]+a[5]*b[4]+a[6]*b[7],  a[4]*b[2]+a[5]*b[5]+a[6]*b[8], a[4]*b[3]+a[5]*b[6]+a[6]*b[9],
			a[7]*b[1]+a[8]*b[4]+a[9]*b[7],  a[7]*b[2]+a[8]*b[5]+a[9]*b[8], a[7]*b[3]+a[8]*b[6]+a[9]*b[9]}
end

local raceHeights={
	{},
	{ -- male
		Human=1.83,
		Gnome=0.88,
		NightElf=2.3,
		Draenei=2.38,
		Dwarf=1.42, 
		Tauren=1.83,
		Orc=1.86,
		Scourge=1.70,
		Troll=1.86,
		BloodElf=1.87,
	},
	{ -- female
		Human=1.76,
		Gnome=0.84,
		NightElf=2.10,
		Draenei=2.25,
		Dwarf=1.36, 
		Tauren=1.93,
		Orc=1.88,
		Scourge=1.68,
		Troll=2.28,
		BloodElf=1.85,
	}
}

local i
function T:New(frame,addon)
	if self ~= T then return end
	local s={}
	
	T:Embed(s)
	
	local gender=UnitSex("player")
	local _,race=UnitRace("player")
	s.playerHeight=raceHeights[gender][race] or raceHeights[2]["Human"]
	
	s.zoneData=addon.zoneData
	s.frameF=768
	s.cameraDistF=1.0
	s.projDist=0.98
	
	s.frame=frame
	
	s.lineTexCache={}
	s.lineCount=0
	s.lastLineCount=0
	
	s.triangleTexCache={}
	s.triangleCount=0
	s.lastTriangleCount=0
	
	s.screenWidth2=1024/2 -- width/2, most of the time needed like that
	s.screenHeight2=768/2
	
	s.playerDirection=0
	s.cameraPitch=0
	s.cameraDistance=50
	s.cameraYaw=0
	s.playerPosX=0
	s.playerPosY=0
	
	s.leftDown=0
	s.rightDown=0
	
	s.drawStart=nil
	
	s.cm={1,0,0,0,1,0,0,0,1} -- camera matrix
	s.icm={nil,nil,nil,nil,nil,nil,nil,nil,nil} -- inverse camera matrix
	
	addon:HookScript(WorldFrame,"OnMouseDown",function(widget,button) s:WorldFrameButton(button,1) end)
	addon:HookScript(WorldFrame,"OnMouseUp",function(widget,button) s:WorldFrameButton(button,0) end)
	
	return s
end

function T:WorldFrameButton(button,state)
	if button=="LeftButton" then self.leftDown=state
	elseif button=="RightButton" then self.rightDown=state end
end

function T:GetCameraHeight()
	return sin(self.cameraPitch)*self.cameraDistance+self.playerHeight
end


function T:InverseProject(screenX,screenY,planeZ)
	local h,sy,sx,sa,a,d,d2,psy,psz,psx,icm
	screenX=(screenX-self.frame:GetLeft())-self.screenWidth2
	screenY=(screenY-self.frame:GetBottom())-self.screenHeight2
	planeZ=planeZ or -self.playerHeight
	
	h=sin(self.cameraPitch)*self.cameraDistance-planeZ
	sy=screenY/self.frameF
	sx=screenX/self.frameF
	sa=atan2(sy,self.projDist)
	a=pi/2.0-self.cameraPitch+sa
	d=h/cos(a)
	if d<0 or d>10000.0 then return nil,nil,nil end
	d2=math.sqrt(self.projDist*self.projDist+sy*sy)
	
	psy=d/d2*self.projDist
	psz=sy/self.projDist*psy
	psx=sx/d2*d
	
	psy=psy-self.cameraDistance
	
	icm=self.icm
	return 	psx*icm[1]+psy*icm[2]+psz*icm[3]+self.playerPosX,
			psx*icm[4]+psy*icm[5]+psz*icm[6]+self.playerPosY,
			psx*icm[7]+psy*icm[8]+psz*icm[9]+self.playerPosZ+self.playerHeight
end

function T:InvertCameraMatrix()
	if self.icm[1]~=nil then return end
	local a,b,c,d,e,f,g,h,i=self.cm[1],self.cm[2],self.cm[3],self.cm[4],self.cm[5],self.cm[6],self.cm[7],self.cm[8],self.cm[9]
	local det = -c*e*g+b*f*g+c*d*h-a*f*h-b*d*i+a*e*i
	
	self.icm[1]=(e*i-f*h)/det
	self.icm[2]=(c*h-b*i)/det
	self.icm[3]=(b*f-c*e)/det
	self.icm[4]=(f*g-d*i)/det
	self.icm[5]=(a*i-c*g)/det
	self.icm[6]=(c*d-a*f)/det
	self.icm[7]=(d*h-e*g)/det
	self.icm[8]=(b*g-a*h)/det
	self.icm[9]=(a*e-b*d)/det
end

function T:GetUnitPosition(unit)	
	local tx,ty,s
	tx,ty=GetPlayerMapPosition(unit)
	if tx==0 and ty==0 then return 0,0 end
	s=self.zoneData:GetCurrentZoneScale()
	if s==nil then return tx*1500,(1-ty)*1000 end
	return tx*s[1],(1-ty)*s[2]
end

--[[
-- simulating a raid environment
local allClasses={"WARRIOR","ROGUE","PRIEST","SHAMAN","DEATHKNIGHT","HUNTER","PALADIN","MAGE","WARLOCK","DRUID"}
local function makeSimMember()
	return {
		x=450+30*math.random(),
		y=90+30*math.random(),
		class=allClasses[math.floor(math.random()*10)+1],
		mx=math.random()*4,
		my=math.random()*4,
		msx=math.random()*2,
		mox=math.random()*10,
		msy=math.random()*2,
		moy=math.random()*10
	}
end
	
local simParty={}
for i=1,4 do
	simParty[i]=makeSimMember()
	simParty[i].name="party"..i
	simParty["party"..i]=simParty[i]
end
for i=1,24 do
	simParty[i]=makeSimMember()
	simParty[i].name="raid"..i
	simParty[i].subGroup=math.floor((i-1)/5)+1
	simParty["raid"..i]=simParty[i]
end

local OriginalUnitClass=UnitClass
function UnitClass(unit)
	if unit=="raid25" then return OriginalUnitClass("player")
	elseif simParty[unit] then return simParty[unit].class,simParty[unit].class
	else return OriginalUnitClass(unit) end
end

local OriginalUnitIsUnit=UnitIsUnit
function UnitIsUnit(u1,u2)
	if (u1=="raid25" and u2=="player") or (u1=="player" and u2=="raid25") then return true
	else return OriginalUnitIsUnit(u1,u2) end
end

function GetNumRaidMembers()
	return 25
end

function IsRaidLeader()
	return true
end

function GetRaidRosterInfo(unit)
	if unit==25 then
		local _,cls=UnitClass("player")
		return UnitName("player"),2,5,80,cls,cls,"",true,false,nil,nil
	elseif simParty[unit] then
		return simParty[unit].name,0,simParty[unit].subGroup,80,simParty[unit].class,simParty[unit].class,"",true,false,nil,nil
	else return nil end
end

local oldUnitPosition=T.GetUnitPosition
function T:GetUnitPosition(unit)
	if simParty then
		if unit=="raid25" then return self:GetUnitPosition("player")
		elseif simParty[unit] then
			local s=simParty[unit]
			return s.x+s.mx*math.sin(GetTime()*s.msx+s.mox), s.y+s.my*math.sin(GetTime()*s.msy+s.moy)
		end
	end
	return oldUnitPosition(self,unit)
end
--]] 

function T:MakeCameraMatrix()
	local adjust,rotZ,rotX,cz,sz,cx,sx
	self.screenWidth2=self.frame:GetWidth()/2
	self.screenHeight2=self.frame:GetHeight()/2	
	
	SaveView(5);
	self.cameraPitch=GetCVar("cameraPitchE")*deg2rad
	self.cameraDistance=GetCVar("cameraDistanceE")*self.cameraDistF
	self.cameraYaw=tonumber(GetCVar("cameraYawE"))*deg2rad
	self.playerDirection=GetPlayerFacing()
	self.frameF=self.screenHeight2*2.0
	self.playerPosX,self.playerPosY=self:GetUnitPosition("player")
	self.playerPosZ=0
	if IsMounted() then self.playerPosZ=self.playerPosZ+0.9 end
		
	adjust=0.0
	if IsMouselooking() or self.leftDown==1 then
		adjust=-self.playerDirection
	end
	
	--print(GetCVar("cameraYawE").." "..(GetPlayerFacing()/deg2rad))
	
	rotZ=-self.playerDirection-adjust-self.cameraYaw
	rotX=self.cameraPitch
		
	cz=cos(rotZ)
	sz=sin(rotZ)
	cx=cos(rotX)
	sx=sin(rotX)

	self.cm[1]=cz
	self.cm[2]=-sz
	self.cm[3]=0
	
	self.cm[4]=cx*sz
	self.cm[5]=cx*cz
	self.cm[6]=-sx

	self.cm[7]=sx*sz
	self.cm[8]=sx*cz
	self.cm[9]=cx
	
	self.icm[1]=nil

end

function T:ProjectMesh(mesh)
	local vs=mesh.vertices
	local v,px,py,pz
	local cm=self.cm
	local cm1,cm2,cm3,cm4,cm5,cm6,cm7,cm8,cm9=cm[1],cm[2],cm[3],cm[4],cm[5],cm[6],cm[7],cm[8],cm[9]
	local cd,pd,ff=self.cameraDistance,self.projDist,self.frameF
	local tx,ty,tz=mesh.translateX-self.playerPosX,
					mesh.translateY-self.playerPosY,
					mesh.translateZ-self.playerPosZ-self.playerHeight
	if mesh.rotateZ~=0.0 or mesh.scaleX~=1.0 or mesh.scaleY~=1.0 or mesh.scaleZ~=1.0 then
		local ca=cos(mesh.rotateZ)
		local sa=sin(mesh.rotateZ)
		local preA,preB,preC,preD=ca*mesh.scaleX,-sa*mesh.scaleX,
							sa*mesh.scaleY, ca*mesh.scaleY
		for i=1,#vs do
			v=vs[i]
			px,py,pz=v[1],v[2],v[3]
			px,py,pz=preA*px+preB*py+tx,	preC*px+preD*py+ty,		mesh.scaleZ*pz+tz
			px,py,pz= px*cm1+py*cm2+pz*cm3,
						px*cm4+py*cm5+pz*cm6+cd, --self.cameraDistance,
						px*cm7+py*cm8+pz*cm9
			-- rotated and translated coordinates
			v[4],v[5],v[6],v[7],v[8]=px,py,pz,
			-- screen coordinates
				px/py*pd*ff,pz/py*pd*ff
		end
							
	else 
		for i=1,#vs do
			v=vs[i]
			px,py,pz=v[1]+tx,v[2]+ty,v[3]+tz
			px,py,pz= px*cm1+py*cm2+pz*cm3,
						px*cm4+py*cm5+pz*cm6+cd, --self.cameraDistance,
						px*cm7+py*cm8+pz*cm9
			-- rotated and translated coordinates
			v[4],v[5],v[6],v[7],v[8]=px,py,pz,
			-- screen coordinates
				px/py*pd*ff,pz/py*pd*ff
		end
		
	end
end

function T:DrawMesh(mesh)
	if not mesh.visible then return end
	local t,l
	local vs=mesh.vertices
	local ts=mesh.triangles
	local ls=mesh.lines
	local r,g,b,a=mesh.r,mesh.g,mesh.b,mesh.a
	
	self:ProjectMesh(mesh)
	for i=1,#ts do		
		t=ts[i]
		if t.visible then
			self:Draw3DTriangle(vs[t.v1],vs[t.v2],vs[t.v3],
							t.a or a,t.r or r,t.g or g,t.b or b)
		end
	end
	for i=1,#ls do
		l=ls[i]
		if l.visible then
			self:Draw3DLine(vs[l.v1],vs[l.v2],
							l.width,l.a or a,l.r or r,l.g or g,l.b or b)
		end
	end
end

do
local C1=127/32768
function T:DrawTriangle(x1,y1,x2,y2,x3,y3,alpha,red,green,blue)
	local tex
		
	local minx=min(x1,x2,x3)
	local miny=min(y1,y2,y3)
	local maxx=max(x1,x2,x3)
	local maxy=max(y1,y2,y3)
	
	if maxx<-self.screenWidth2 then return
	elseif minx>self.screenWidth2 then return
	elseif maxy<-self.screenHeight2 then return
	elseif miny>self.screenHeight2 then return
	end
	
	-- screen y coordinates increases up, texture increases down so y gets reversed in ty
	-- also need to take this into account when finding the corner later
	local dx=maxx-minx
	local dy=miny-maxy
	if dx==0 or dy==0 then return end

	local tx1,tx2,tx3,ty1,ty2,ty3=
		(x1-minx)/dx,
		(x2-minx)/dx,
		(x3-minx)/dx,
		(y1-maxy)/dy,
		(y2-maxy)/dy,
		(y3-maxy)/dy
	local r=0

	if     x1==minx and y1==maxy then 
	elseif x1==maxx and y1==maxy then
		r=1
		tx1,ty1,tx2,ty2,tx3,ty3=ty1,1-tx1,ty2,1-tx2,ty3,1-tx3
	elseif x1==maxx and y1==miny then
		r=2
		tx1,ty1,tx2,ty2,tx3,ty3=1-tx1,1-ty1,1-tx2,1-ty2,1-tx3,1-ty3
	elseif x1==minx and y1==miny then
		r=3
		tx1,ty1,tx2,ty2,tx3,ty3=1-ty1,tx1,1-ty2,tx2,1-ty3,tx3

	elseif x2==minx and y2==maxy then 
		tx1,ty1,tx2,ty2=tx2,ty2,tx1,ty1
	elseif x2==maxx and y2==maxy then
		r=1
		tx1,ty1,tx2,ty2,tx3,ty3=ty2,1-tx2,ty1,1-tx1,ty3,1-tx3
	elseif x2==maxx and y2==miny then
		r=2
		tx1,ty1,tx2,ty2,tx3,ty3=1-tx2,1-ty2,1-tx1,1-ty1,1-tx3,1-ty3
	elseif x2==minx and y2==miny then
		r=3
		tx1,ty1,tx2,ty2,tx3,ty3=1-ty2,tx2,1-ty1,tx1,1-ty3,tx3

	elseif x3==minx and y3==maxy then 
		tx1,ty1,tx3,ty3=tx3,ty3,tx1,ty1
	elseif x3==maxx and y3==maxy then
		r=1
		tx1,ty1,tx2,ty2,tx3,ty3=ty3,1-tx3,ty2,1-tx2,ty1,1-tx1
	elseif x3==maxx and y3==miny then
		r=2
		tx1,ty1,tx2,ty2,tx3,ty3=1-tx3,1-ty3,1-tx2,1-ty2,1-tx1,1-ty1
	elseif x3==minx and y3==miny then
		r=3
		tx1,ty1,tx2,ty2,tx3,ty3=1-ty3,tx3,1-ty2,tx2,1-ty1,tx1
	end
	
	if tx2==0 then
		tx2,ty2,tx3,ty3=tx3,ty3,tx2,ty2
		if tx2==0 then return end
	end

	local det=32768/32512*(tx2*ty3-tx3*ty2)
	if abs(det)<0.0001 then return end

	--tex=self:ReuseTriangleTexture()
	self.triangleCount=self.triangleCount+1
	if self.triangleCount<=#self.triangleTexCache then
		tex=self.triangleTexCache[self.triangleCount]
	else
		tex=self.frame:CreateTexture("AVR_TRIANGLE_"..self.triangleCount,"ARTWORK")
		tex:SetTexture("Interface\\AddOns\\AVR\\Textures\\triangle")
		insert(self.triangleTexCache,tex)
	end
	tex:Hide()
		
	tex:SetPoint("BOTTOMLEFT",self.frame,"CENTER",minx,miny)
	tex:SetPoint("TOPRIGHT",self.frame,"CENTER",maxx,maxy)
		
--	local kx=tx3
--	local sx=tx2
--	local ky=ty2/tx2
--	local sy=ty3-tx3*ky
--	m={256/254,0,-1/256, 0,256/254,-1/256, 0,0,1 } -- remove one pixel edge from texture
--	m=matrixMultiply({sx,kx,0, 0,1,0, 0,0,1},m) -- shear and scale one corner in place
--	m=matrixMultiply({1,0,0, ky,sy,0, 0,0,1},m) -- shear and scale other corner in place
--	if r==1 then m=matrixMultiply({0,-1,1, 1,0,0, 0,0,1},m) -- rotate and translate as needed
--	elseif r==2 then m=matrixMultiply({-1,0,1, 0,-1,1, 0,0,1},m)
--	elseif r==3 then m=matrixMultiply({0,1,0, -1,0,1, 0,0,1},m)
--	end
--	setCoords(tex,m[1],m[2],m[3],m[4],m[5],m[6])

	
--	All the above matrix multiplications done and simplified the equations a great deal
	tx2,tx3,ty2,ty3=tx2/det,tx3/det,ty2/det,ty3/det
	if r==0 then 
		tex:SetTexCoord( 	C1, 				C1,
							-tx3+C1, 			tx2+C1,
							ty3+C1, 			-ty2+C1,
							ty3-tx3+C1, 		-ty2+tx2+C1 )
	elseif r==1 then 
		tex:SetTexCoord( 	-tx3+C1, 			tx2+C1,
							ty3-tx3+C1, 		-ty2+tx2+C1,
							C1, 				C1,
							ty3+C1, 			-ty2+C1)
	elseif r==2 then 
		tex:SetTexCoord( 	ty3-tx3+C1, 		-ty2+tx2+C1,
							ty3+C1, 			-ty2+C1,
							-tx3+C1, 			tx2+C1,
							C1, 				C1)
	else 
		tex:SetTexCoord( 	ty3+C1, 			-ty2+C1,
							C1, 				C1,
							ty3-tx3+C1, 		-ty2+tx2+C1,
							-tx3+C1, 			tx2+C1)
	end

	tex:SetVertexColor(red,green,blue,alpha)
	tex:Show()
	return tex
end
end

function T:DrawLine(sx,sy,ex,ey,lineW,lineAlpha,r,g,b)
	if sx==ex and sy==ey then return nil end
	local dx,dy=ex-sx,ey-sy
	local w,h=abs(dx),abs(dy)
	local d,tex

	if w>h then d=w
	else d=h end
	
--	tex=self:ReuseLineTexture()
	self.lineCount=self.lineCount+1
	if self.lineCount<=#self.lineTexCache then
		tex=self.lineTexCache[self.lineCount]
	else
		tex=self.frame:CreateTexture("AVR_LINE_"..self.lineCount,"ARTWORK")
		tex:SetTexture("Interface\\AddOns\\AVR\\Textures\\line")
		insert(self.lineTexCache,tex)
	end
	tex:Hide()


	local tx=(sx+ex-d)/2.0
	local ty=(sy+ey-d)/2.0
	local a=atan2(dy,dx)
	
	if lineW==0.0 then
		lineW=1.0
	else
		if lineW<1.0 then
			lineAlpha=lineAlpha*lineW
			lineW=1.0
		end
	end
	local s=lineW*16/d	
--	local ca=cos(a)*s -- multiply with s already, the transform matrix needs it
--	local sa=sin(a)*s
	local ca=cos(a)/s -- divide by s already so we can ignore determinant in the matrix later
	local sa=sin(a)/s
		
	tex:SetPoint("BOTTOMLEFT",self.frame,"CENTER",tx,ty)
	tex:SetPoint("TOPRIGHT",self.frame,"CENTER",tx+d,ty+d)
--	setCoords(tex, ca,sa,-ca/2.0-sa/2.0+0.5,
--	              -sa,ca, sa/2.0-ca/2.0+0.5)

--	local A,B,C,D,E,F=	 ca,sa,-ca/2.0-sa/2.0+0.5,
--						-sa,ca,sa/2.0-ca/2.0+0.5
--	local det = s*s -- A*E - B*D;

--[[
	B*F-C*E
	=sa*(sa/2.0-ca/2.0+0.5)-(-ca/2.0-sa/2.0+0.5)*ca
	=sa*sa/2.0+sa*0.5+ca*ca/2.0-ca*0.5
	=(s*s+sa-ca)/2.0
	
	-(A*F)+C*D
	=-(ca*(sa/2.0-ca/2.0+0.5))+(-ca/2.0-sa/2.0+0.5)*(-sa)
	=ca*ca/2.0-ca*0.5+sa*sa/2.0-sa*0.5
	=(s*s-sa-ca)/2.0
]]
--[[[
	local ULx, ULy, LLx, LLy, URx, URy, LRx, LRy;
	ULx, ULy = ( B*F - C*E ) / det, ( -(A*F) + C*D ) / det;
	LLx, LLy = ( -B + B*F - C*E ) / det, ( A - A*F + C*D ) / det;
	URx, URy = ( E + B*F - C*E ) / det, ( -D - A*F + C*D ) / det;
	LRx, LRy = ( E - B + B*F - C*E ) / det, ( -D + A -(A*F) + C*D ) / det;
--	local mi=min(ULx,ULy,LLx,LLy,URx,URy,LRx,LRy)
--	local ma=max(ULx,ULy,LLx,LLy,URx,URy,LRx,LRy)
--	if mi<-1000 then print("min="..mi) end
--	if ma>1000 then print("max="..ma) end
	tex:SetTexCoord(ULx, ULy, LLx, LLy, URx, URy, LRx, LRy);
]]
	local C1,C2=(1+sa-ca)/2.0, (1-sa-ca)/2.0
	tex:SetTexCoord( 	C1, 		C2,
						-sa+C1, 	ca+C2,
						ca+C1, 		sa+C2,
						ca-sa+C1, 	ca+sa+C2)

	tex:SetVertexColor(r,g,b,lineAlpha)
	
	tex:Show()
	return tex
end

function T:ClipToProj(v1,v2) -- v1[5]<self.projDist<=v2[5]
	local s=(v2[5]-self.projDist)/(v2[5]-v1[5])
	return (v2[4]+(v1[4]-v2[4])*s)*self.frameF,
			(v2[6]+(v1[6]-v2[6])*s)*self.frameF,
			self.projDist
end

function T:Draw3DLine(v1,v2,width,a,r,g,b)
	-- clip to projection plane if needed
	if v1[5]<self.projDist then
		if v2[5]<self.projDist then return end
		local sx,sy,py=self:ClipToProj(v1,v2)
		width=width/((py+v2[5])/2)*self.projDist
		self:DrawLine(sx,sy,v2[7],v2[8],width,a,r,g,b)		
	elseif v2[5]<self.projDist then
		local sx,sy,py=self:ClipToProj(v2,v1)
		width=width/((py+v1[5])/2)*self.projDist
		self:DrawLine(v1[7],v1[8],sx,sy,width,a,r,g,b)		
	else
		width=width/((v1[5]+v2[5])/2)*self.projDist
		self:DrawLine(v1[7],v1[8],v2[7],v2[8],width,a,r,g,b)
	end
end

function T:Draw3DTriangle(v1,v2,v3,a,r,g,b)
	local z1,z2,z3=v1[5],v2[5],v3[5]
	local d=self.projDist

	-- clip to projection plane if needed
	if z1<d then
		if z2<d then
			if z3<d then -- all behind
				return 
			else -- 1 and 2 behind
				local sx1,sy1,_=self:ClipToProj(v1,v3)
				local sx2,sy2,_=self:ClipToProj(v2,v3)
				self:DrawTriangle(v3[7],v3[8],sx1,sy1,sx2,sy2,a,r,g,b)
			end
		elseif z3<d then -- 1 and 3 behind
			local sx1,sy1,_=self:ClipToProj(v1,v2)
			local sx2,sy2,_=self:ClipToProj(v3,v2)
			self:DrawTriangle(v2[7],v2[8],sx1,sy1,sx2,sy2,a,r,g,b)
		else -- 1 behind
			local sx1,sy1,_=self:ClipToProj(v1,v2)
			local sx2,sy2,_=self:ClipToProj(v1,v3)
			self:DrawTriangle(sx1,sy1,v2[7],v2[8],v3[7],v3[8],a,r,g,b)
			self:DrawTriangle(sx1,sy1,v3[7],v3[8],sx2,sy2,a,r,g,b)
		end
	elseif z2<d then
		if z3<d then -- 2 and 3 behind
			local sx1,sy1,_=self:ClipToProj(v2,v1)
			local sx2,sy2,_=self:ClipToProj(v3,v1)
			self:DrawTriangle(v1[7],v1[8],sx1,sy1,sx2,sy2,a,r,g,b)
		else -- 2 behind
			local sx1,sy1,_=self:ClipToProj(v2,v1)
			local sx2,sy2,_=self:ClipToProj(v2,v3)
			self:DrawTriangle(sx1,sy1,v1[7],v1[8],v3[7],v3[8],a,r,g,b)
			self:DrawTriangle(sx1,sy1,v3[7],v3[8],sx2,sy2,a,r,g,b)
		end
	elseif z3<d then -- 3 behind
		local sx1,sy1,_=self:ClipToProj(v3,v1)
		local sx2,sy2,_=self:ClipToProj(v3,v2)
		self:DrawTriangle(sx1,sy1,v1[7],v1[8],v2[7],v2[8],a,r,g,b)
		self:DrawTriangle(sx1,sy1,v2[7],v2[8],sx2,sy2,a,r,g,b)	
	else -- all in front
		self:DrawTriangle(v1[7],v1[8],v2[7],v2[8],v3[7],v3[8],a,r,g,b)
	end	
end

function T:ClearAllTriangles()
	local tex
	for i=1,#self.triangleTexCache do
		self.triangleTexCache[i]:Hide()
	end
	self.triangleCount=0
	self.LastTriangleCount=0
end

function T:ClearAllLines()
	local tex
	for i=1,#self.lineTexCache do
		self.lineTexCache[i]:Hide()
	end
	self.lineCount=0
	self.lastLineCount=0
end

function T:StartRender()
	self:MakeCameraMatrix()
	self.lastLineCount=self.lineCount
	self.lastTriangleCount=self.triangleCount
	self.lineCount=0
	self.triangleCount=0
end
function T:EndRender()
	local tex
	for i=self.lineCount+1,self.lastLineCount do
		self.lineTexCache[i]:Hide()
	end
	for i=self.triangleCount+1,self.lastTriangleCount do
		self.triangleTexCache[i]:Hide()
	end
end
