local ADDON_NAME="AVR"
local L = LibStub("AceLocale-3.0"):GetLocale(ADDON_NAME, true)

local _G = _G
local sin = math.sin
local cos = math.cos
local pi = math.pi

local Core=AVR

local unpackDouble=AVRUnpackDouble
local packDouble=AVRPackDouble

AVRMarkerMesh={Embed=Core.Embed}
AVRMarkerMesh.meshInfo={
	class="AVRMarkerMesh",
	derived=false,
	guiCreateNew=false,
	guiName=nil,
	receivable=true
}
AVRMarkerMesh.class="AVRMarkerMesh"
function AVRMarkerMesh:New(radius,spokes)
	if self ~= AVRMarkerMesh then return end
	local s=AVRMesh:New()
	AVRMarkerMesh:Embed(s)
	s.class=AVRMarkerMesh.class
	s.radius=radius or 1
	s.spokes=spokes or 6
	s.classColor=false
	s.vertices=nil
	s.line=false
	s.lineVertex=nil
	s.name="Marker"
	return s
end

function AVRMarkerMesh:GetOptions()
	local o=AVRMesh.GetOptions(self)
	o.args.marker = {
		type = "group",
		name = L["Marker properties"],
		inline = true,
		order = 80,
		args = {
			radius = {
				type = "range",
				name = L["Radius"],
				order = 10,
				width = "full",
				min = 0.0, max=10, bigStep=0.5
			},					
			spokes = {
				type = "range",
				name = L["Spokes"],
				order = 20,
				width = "full",
				min = 4, max=10, step=2.0
			},
			classColor = {
				type = "toggle",
				name = L["Class color"],
				order = 30,
				width = "full"
			},
			line = {
				type = "toggle",
				name = L["Draw line"],
				order = 40,
				width = "full"
			},
		}
	}
	return o
end

function AVRMarkerMesh:SetLine(value)
	self.line=value
	self.vertices=nil
end
function AVRMarkerMesh:GetLine()
	return self.line
end

function AVRMarkerMesh:SetRadius(radius)
	self.radius=radius
	self.vertices=nil
end
function AVRMarkerMesh:GetRadius()
	return self.radius
end

function AVRMarkerMesh:SetSpokes(spokes)
	self.spokes=spokes
	self.vertices=nil
end
function AVRMarkerMesh:GetSpokes()
	return self.spokes
end

function AVRMarkerMesh:SetClassColor(value)
	self.classColor=value
end
function AVRMarkerMesh:GetClassColor()
	return self.classColor
end

function AVRMarkerMesh:Pack()
	local s=AVRMesh.Pack(self)
	s.rad=self.radius
	s.spo=self.spokes
	s.clc=self.classColor
	s.lin=self.line
	return s
end

function AVRMarkerMesh:Unpack(s)
	AVRMesh.Unpack(self,s)
	self.radius=s.rad or 1
	self.spokes=s.spo or 6
	if s.clc==nil then self.classColor=false
	else self.classColor=s.clc end
	if s.lin==nil then self.line=false
	else self.line=s.lin end
	self.vertices=nil
end

function AVRMarkerMesh:GenerateMesh()
	local a
	local ox=self.radius
	local oy=0
	local x,y
	
	for i=1,self.spokes do
		a=pi*2/self.spokes*i
		
		x=cos(a)*self.radius
		y=sin(a)*self.radius
		
		if (i%2)==0 then
			self:AddTriangle(0,0,0,ox,oy,0,x,y,0,nil,0,0,0)
		else
			self:AddTriangle(0,0,0,ox,oy,0,x,y,0)
		end
		
		ox,oy=x,y
	end
	
	if self.line then
		self.lineVertex=self:AddVertex(0,0,0)
		local l=AVRLine:New(1,self.lineVertex,128)
		self:AddLine(l)
	end
	
	AVRMesh.GenerateMesh(self)
end

function AVRMarkerMesh:OnUpdate(threed)
	if self.classColor then
		local cls=nil
		if self.followUnit then
			_,cls=UnitClass(self.followUnit)
		elseif self.followPlayer then
			_,cls=UnitClass("player")
		end
		if cls then
			local c=RAID_CLASS_COLORS[cls]
			if c then self:SetColor(c.r,c.g,c.b) 
			else self:SetColor(1.0,1.0,1.0) end
		end
	end
	if self.line then
		if self.vertices==nil or self.lines==nil or self.triangles==nil then
			self.vertices={}
			self.lines={}
			self.triangles={}
			self:GenerateMesh()
		end
	
		local unit=(self.followUnit or (self.followPlayer and "player") or nil)
		local px,py=threed:GetUnitPosition("player")
		local v=self.vertices[self.lineVertex]
		v[1]=nil
		if px~=0 or py~=0 then
			if unit then
				local ux,uy=threed:GetUnitPosition(unit)
				if ux~=0 or uy~=0 then 
					v[1]=px-ux
					v[2]=py-uy
				end
			else
				v[1]=px
				v[2]=py
			end
		end
		if v[1]==nil then
			v[1]=self.vertices[1][1]
			v[2]=self.vertices[1][2]
		end
	end
	AVRMesh.OnUpdate(self,threed)
end

AVR:RegisterMeshClass(AVRMarkerMesh)
